import { WellKnownMimeType, encodeRoute, encodeBearerAuthMetadata, encodeSimpleAuthMetadata, encodeCompositeMetadata, encodeAndAddWellKnownMetadata, CompositeMetadata, decodeRoutes } from '@rsocket/composite-metadata';
import { Buffer as Buffer$1 } from 'buffer';
import { deserializeFrame, FrameTypes, ErrorCodes, RSocketConnector } from '@rsocket/core';
export { ErrorCodes, RSocketError } from '@rsocket/core';
import * as i0 from '@angular/core';
import { Injectable, inject } from '@angular/core';
import { ReplaySubject, BehaviorSubject, retry, defer, first, map, switchMap } from 'rxjs';
import { WebsocketClientTransport } from '@rsocket/websocket-client';
import { print } from 'graphql/index';

class RsocketConfig {
    constructor(url, route = 'route', config = {}) {
        this.url = url;
        this.route = route;
        this.config = config;
        this.payload({});
    }
    payload({ data, auth }) {
        if (!this.config.setup) {
            this.config['setup'] = {};
        }
        this.config.setup['payload'] = {
            data: data ? Buffer$1.from(JSON.stringify(data)) : undefined,
            metadata: this.buildAuthMetadata({ route: this.route, auth })
        };
        return this;
    }
    setAuth(auth) {
        if (!this.config.setup?.payload?.metadata) {
            throw new Error('must be call payload first');
        }
        this.config.setup.payload.metadata = this.buildAuthMetadata({ route: this.route, auth });
    }
    buildAuthMetadata({ route, auth }) {
        const map = new Map();
        map.set(WellKnownMimeType.MESSAGE_RSOCKET_ROUTING, encodeRoute(route));
        if (auth) {
            const { token, simple } = auth;
            if (token) {
                map.set(WellKnownMimeType.MESSAGE_RSOCKET_AUTHENTICATION, encodeBearerAuthMetadata(token));
            }
            else if (simple) {
                map.set(WellKnownMimeType.MESSAGE_RSOCKET_AUTHENTICATION, encodeSimpleAuthMetadata(simple.username, simple.password));
            }
        }
        return encodeCompositeMetadata(map);
    }
    getUrl() {
        return this.url;
    }
    getConfig() {
        return this.config;
    }
}

var ConnectionStatus;
(function (ConnectionStatus) {
    ConnectionStatus[ConnectionStatus["DISCONNECTED"] = 0] = "DISCONNECTED";
    ConnectionStatus[ConnectionStatus["REJECT"] = 1] = "REJECT";
    ConnectionStatus[ConnectionStatus["CONNECTING"] = 2] = "CONNECTING";
    ConnectionStatus[ConnectionStatus["CONNECTED"] = 3] = "CONNECTED";
})(ConnectionStatus || (ConnectionStatus = {}));
var NetWorkStatus;
(function (NetWorkStatus) {
    NetWorkStatus[NetWorkStatus["IDLE"] = 0] = "IDLE";
    NetWorkStatus[NetWorkStatus["UPLOADING"] = 1] = "UPLOADING";
    NetWorkStatus[NetWorkStatus["DOWNLOADING"] = 2] = "DOWNLOADING";
    NetWorkStatus[NetWorkStatus["DONE"] = 3] = "DONE";
})(NetWorkStatus || (NetWorkStatus = {}));

let websocket;
const rsocketCreator = (url, { onSend, onMessage, onResumeOk, onRejectSetup, onResumeReject }) => {
    websocket?.close();
    websocket = new WebSocket(url);
    const originalSend = websocket.send;
    let resumeTimeout;
    websocket.send = (data) => {
        let frame = deserializeFrame(data);
        if (frame.type === FrameTypes.RESUME) {
            resumeTimeout = window.setTimeout(onResumeReject, 5000);
        }
        sendHandler(data);
        websocket.send = sendHandler;
    };
    const sendHandler = (data) => {
        onSend();
        originalSend.call(websocket, data);
    };
    function onFirstMessage(message) {
        const buffer = Buffer.from(message.data);
        let frame = deserializeFrame(buffer);
        switch (frame.type) {
            case FrameTypes.ERROR:
                switch (frame.code) {
                    case ErrorCodes.REJECTED_SETUP:
                        window.clearTimeout(resumeTimeout);
                        onRejectSetup();
                        break;
                    case ErrorCodes.REJECTED_RESUME:
                        window.clearTimeout(resumeTimeout);
                        onResumeReject();
                        break;
                }
                break;
            case FrameTypes.RESUME_OK:
                window.clearTimeout(resumeTimeout);
                onResumeOk();
                break;
        }
        websocket.removeEventListener('message', onFirstMessage);
    }
    const closeHandler = () => {
        websocket.removeEventListener('message', onMessage);
        websocket.removeEventListener('message', onFirstMessage);
        websocket.removeEventListener('close', closeHandler);
    };
    websocket.addEventListener('message', onMessage);
    websocket.addEventListener('message', onFirstMessage);
    websocket.addEventListener('close', closeHandler);
    return websocket;
};

// Borrowed from https://github.com/ai/nanoid/blob/main/non-secure/index.js
// This alphabet uses `A-Za-z0-9_-` symbols. A genetic algorithm helped
// optimize the gzip compression for this alphabet.
const urlAlphabet = 'useandom-26T198340PX75pxJACKVERYMINDBUSHWOLF_GQZbfghjklqvwyzrict';
const nanoid = (size = 21) => {
    let id = '';
    // A compact alternative for `for (var i = 0; i < step; i++)`.
    let i = size;
    while (i--) {
        // `| 0` is more compact and faster than `Math.floor()`.
        id += urlAlphabet[(Math.random() * 64) | 0];
    }
    return id;
};

class RsocketClient {
    constructor() {
        this._rsocketClient = new ReplaySubject(1);
        this._error = new ReplaySubject(1);
        this._connectionStatus = new BehaviorSubject(ConnectionStatus.DISCONNECTED);
        this._networkStatus = new BehaviorSubject(NetWorkStatus.IDLE);
        this.rsocketConfig = undefined;
    }
    connect() {
        this._connectionStatus.pipe(first()).subscribe(connectionStatus => {
            if (![ConnectionStatus.REJECT, ConnectionStatus.DISCONNECTED].includes(this._connectionStatus.getValue())) {
                return;
            }
            this._connectionStatus.next(ConnectionStatus.CONNECTING);
            if (!this.rsocketConfig) {
                throw new Error("rsocketConfig config is null");
            }
            const config = this.rsocketConfig.getConfig();
            let isRejectSetup = false;
            config['resume'] = {
                tokenGenerator: () => Buffer.from(nanoid()),
                reconnectFunction: (a) => new Promise((resolve, reject) => {
                    if (isRejectSetup) {
                        reject();
                    }
                    console.log(`resume: delay ${2000}`);
                    setTimeout(resolve, 2000, 100);
                }),
            };
            config['transport'] = new WebsocketClientTransport({
                url: this.rsocketConfig.getUrl(),
                wsCreator: (url) => rsocketCreator(url, {
                    onSend: () => this._networkStatus.next(NetWorkStatus.UPLOADING),
                    onMessage: () => this._networkStatus.next(NetWorkStatus.DOWNLOADING),
                    onResumeOk: () => {
                        isRejectSetup = false;
                        if (this._connectionStatus.getValue() !== ConnectionStatus.CONNECTED) {
                            this._connectionStatus.next(ConnectionStatus.CONNECTED);
                        }
                    },
                    onResumeReject: () => {
                        isRejectSetup = false;
                        connector();
                    },
                    onRejectSetup: () => {
                        isRejectSetup = true;
                    }
                })
            });
            const connector = () => defer(() => new RSocketConnector(config).connect()).pipe(retry({ delay: 2000 })).subscribe({
                next: (rsocketClient) => {
                    console.log('Connected to RSocket server');
                    rsocketClient.onClose(error => {
                        const rsocketError = error;
                        console.log(rsocketError);
                        if (rsocketError) {
                            this._error.next(rsocketError);
                            if (rsocketError.code === ErrorCodes.REJECTED_SETUP) {
                                this._connectionStatus.next(ConnectionStatus.REJECT);
                            }
                            else {
                                this._connectionStatus.next(ConnectionStatus.DISCONNECTED);
                            }
                        }
                    });
                    this._rsocketClient.next(rsocketClient);
                },
                error: (error) => {
                    console.error('Failed to connect to RSocket server', error);
                }
            });
            connector();
        });
    }
    encodePayload({ route, data, encodeData = true }) {
        let dataBuffer = null;
        if (data) {
            if (encodeData) {
                dataBuffer = Buffer.from(JSON.stringify(data));
            }
            else {
                dataBuffer = Buffer.from(data);
            }
        }
        return {
            data: dataBuffer,
            metadata: encodeAndAddWellKnownMetadata(Buffer.alloc(0), WellKnownMimeType.MESSAGE_RSOCKET_ROUTING, encodeRoute(route))
        };
    }
    fireAndForget({ payload }) {
        this._rsocketClient.pipe(map(rSocket => rSocket.fireAndForget(this.encodePayload(payload), {
            onError: (error) => {
            },
            onComplete: () => {
            }
        })), first()).subscribe(() => {
            this._networkStatus.next(NetWorkStatus.IDLE);
        });
    }
    requestResponse({ payload, responderStream }) {
        return this._rsocketClient.pipe(map(rSocket => rSocket.requestResponse(this.encodePayload(payload), responderStream)), first());
    }
    requestStream({ payload, initialRequestN, responderStream }) {
        return this._rsocketClient.pipe(map(rSocket => {
            return rSocket.requestStream(this.encodePayload(payload), initialRequestN, responderStream);
        }), first());
    }
    requestChannel({ payload, initialRequestN, isCompleted, responderStream }) {
        return this._rsocketClient.pipe(map(rSocket => rSocket.requestChannel(this.encodePayload(payload), initialRequestN, isCompleted, responderStream)), first());
    }
    get error() {
        return this._error.asObservable();
    }
    setRsocketConfig(rsocketConfig) {
        this.rsocketConfig = rsocketConfig;
    }
    get connectionStatus() {
        return this._connectionStatus.asObservable();
    }
    updateNetworkState(state) {
        this._networkStatus.next(state);
    }
    get networkStatus() {
        return this._networkStatus.asObservable();
    }
    setConnected() {
        this._connectionStatus.next(ConnectionStatus.CONNECTED);
    }
    setAuth(auth) {
        if (!this.rsocketConfig) {
            throw new Error("rsocketConfig is undefined.");
        }
        this.rsocketConfig.setAuth(auth);
    }
}

class RsocketService {
    constructor() {
        this.rsocketClient = new RsocketClient();
    }
    connect() {
        this.rsocketClient.connect();
    }
    get connectionStatus() {
        return this.rsocketClient.connectionStatus;
    }
    get networkStatus() {
        return this.rsocketClient.networkStatus;
    }
    fireAndForget(payload) {
        this.rsocketClient.fireAndForget(payload);
    }
    requestChannel(params) {
        return this.rsocketClient.requestChannel(params);
    }
    setConfig(config) {
        this.rsocketClient.setRsocketConfig(config);
    }
    setAuth(auth) {
        this.rsocketClient.setAuth(auth);
    }
    setConnected() {
        this.rsocketClient.setConnected();
    }
    get error() {
        return this.rsocketClient.error;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.3.3", ngImport: i0, type: RsocketService, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "17.3.3", ngImport: i0, type: RsocketService, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.3.3", ngImport: i0, type: RsocketService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [] });

const routeHandleMap = new Map();
function mapMetaData(metadata) {
    const mappedMetaData = new Map();
    if (metadata) {
        for (let metaData of new CompositeMetadata(metadata)) {
            switch (metaData.mimeType) {
                case WellKnownMimeType.MESSAGE_RSOCKET_ROUTING.string: {
                    const tags = [];
                    let routes = decodeRoutes(metaData.content);
                    let entry = routes.next();
                    while (!entry.done) {
                        tags.push(entry.value);
                        entry = routes.next();
                    }
                    const joinedRoute = tags.join(".");
                    mappedMetaData.set(WellKnownMimeType.MESSAGE_RSOCKET_ROUTING.string, joinedRoute);
                }
            }
        }
    }
    return mappedMetaData;
}
function registerRoute({ route, type = 'json', handle }) {
    switch (type) {
        case 'string':
            routeHandleMap.set(route, (payload) => handle(payload.data?.toString()));
            break;
        case 'json':
            routeHandleMap.set(route, (payload) => handle(payload.data));
    }
}
function deleteRoute(route) {
    if (typeof route === 'string') {
        routeHandleMap.delete(route);
    }
    else {
        Array.from(routeHandleMap.keys()).forEach(key => {
            if (route.test(key)) {
                routeHandleMap.delete(key);
            }
        });
    }
}
function handleFireAndForget(payload) {
    const route = mapMetaData(payload.metadata).get(WellKnownMimeType.MESSAGE_RSOCKET_ROUTING.string);
    const handle = routeHandleMap.get(route);
    if (!handle) {
        console.error(`can not find route for ${route}`);
    }
    else {
        handle(payload);
    }
}

class GraphqlClient {
    constructor(rsocketClient, route) {
        this.rsocketClient = rsocketClient;
        this.route = route;
    }
    query(query) {
        const { documentNode, extract, variables } = query;
        const result = new ReplaySubject(1);
        return this.rsocketClient.requestResponse({
            payload: { data: { query: print(documentNode), variables }, route: this.route },
            responderStream: {
                onNext: (payload, isComplete) => {
                    result.next(extract(payload));
                    this.rsocketClient.updateNetworkState(NetWorkStatus.DONE);
                },
                onComplete() {
                },
                onError(error) {
                    console.log(error);
                },
                onExtension(extendedType, content, canBeIgnored) {
                }
            }
        }).pipe(first(), switchMap(() => result));
    }
    subscription(subscription) {
        const { documentNode, extract, variables } = subscription;
        const result = new ReplaySubject();
        const isCompleted = new BehaviorSubject(false);
        const error = new ReplaySubject();
        return this.rsocketClient.requestStream({
            payload: { data: { query: print(documentNode), variables }, route: this.route },
            initialRequestN: 1,
            responderStream: {
                onNext: (payload, isComplete) => {
                    result.next(extract(payload));
                },
                onComplete: () => {
                    this.rsocketClient.updateNetworkState(NetWorkStatus.DONE);
                },
                onError(e) {
                    error.next(e);
                },
                onExtension(extendedType, content, canBeIgnored) {
                }
            }
        }).pipe(map(requester => {
            return { result, requester, isCompleted, error };
        }));
    }
    mutation(mutation) {
        const { documentNode, extract, variables } = mutation;
        const result = new ReplaySubject(1);
        return this.rsocketClient.requestResponse({
            payload: { data: { query: print(documentNode), variables }, route: this.route },
            responderStream: {
                onNext: (payload, isComplete) => {
                    result.next(extract(payload));
                    this.rsocketClient.updateNetworkState(NetWorkStatus.DONE);
                },
                onComplete() {
                },
                onError(error) {
                    console.log(error);
                },
                onExtension(extendedType, content, canBeIgnored) {
                }
            }
        }).pipe(first(), switchMap(() => result), first());
    }
}

class GraphqlService {
    constructor() {
        this._graphqlClient = {};
        this.rsocketService = inject(RsocketService);
        this._graphqlClient = new GraphqlClient(this.rsocketService.rsocketClient, 'graphql');
    }
    query(query) {
        return this._graphqlClient.query(query);
    }
    subscription(subscription) {
        return this._graphqlClient.subscription(subscription);
    }
    mutation(mutation) {
        return this._graphqlClient.mutation(mutation);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.3.3", ngImport: i0, type: GraphqlService, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "17.3.3", ngImport: i0, type: GraphqlService, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.3.3", ngImport: i0, type: GraphqlService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [] });

var Operation;
(function (Operation) {
    Operation["QUERY"] = "query";
    Operation["SUBSCRIPTION"] = "subscription";
    Operation["MUTATION"] = "mutation";
})(Operation || (Operation = {}));
function create(documentNode, extract, operation, variables) {
    const valid = documentNode.definitions.some(definition => {
        //@ts-ignore
        return definition.selectionSet.selections.some(selection => {
            if ("name" in selection && selection.name.value === extract) {
                //@ts-ignore
                if (definition.operation !== operation) {
                    throw Error(`createGraphqlExtractor=> '${operation}' type error ${documentNode.loc?.source.body}`);
                }
                return true;
            }
            return false;
        });
    });
    if (!valid) {
        throw Error(`createGraphqlExtractor=> '${extract}' not in ${documentNode.loc?.source.body}`);
    }
    return {
        documentNode,
        extract: (payload) => {
            if (!payload.data) {
                throw Error(`GraphqlExtractor => payload.data is null, extract: ${extract}`);
            }
            const value = JSON.parse(payload.data.toString());
            if (value['data'] !== undefined && value['data'][extract] !== undefined) {
                return value['data'][extract];
            }
            throw Error(`GraphqlExtractor => 'data' or '${extract}' not in ${JSON.stringify(value)}`);
        },
        variables
    };
}
function createQuery(documentNode, extract, variables) {
    return create(documentNode, extract, Operation.QUERY, variables);
}
function createSubscription(documentNode, extract, variables) {
    return create(documentNode, extract, Operation.SUBSCRIPTION, variables);
}
function createMutation(documentNode, extract, variables) {
    return create(documentNode, extract, Operation.MUTATION, variables);
}

/*
 * Public API Surface of ngx-rsocket-graphql
 */
window.Buffer = Buffer$1;

/**
 * Generated bundle index. Do not edit.
 */

export { ConnectionStatus, GraphqlService, NetWorkStatus, RsocketConfig, RsocketService, createMutation, createQuery, createSubscription, deleteRoute, handleFireAndForget, registerRoute };
//# sourceMappingURL=ngx-rsocket-graphql.mjs.map
